import sqlite3
from urllib.parse import parse_qs

#
# příprava odpovědi
#

# získání řádek tabulky v <tbody> vyhovujících dotazu
def get_monitors(brand, support, connection=None):

    if connection is None:
        connection = sqlite3.connect("./monitory.sqlite")
        local_connection_flag = True
    else:
        local_connection_flag = False

    # HTML pro <tbody>
    html = ''

    # připojení k SQLite-databázi
    cursor = connection.cursor()

    # dotaz do databáze
    if support == '':
        cursor.execute(f"""
            SELECT * FROM monitory
                WHERE brand LIKE '%{brand}%';
        """)
    else:
        cursor.execute(f"""
            SELECT * FROM monitory
                WHERE brand LIKE '%{brand}%' AND support = "{support}";
        """)
    # kurzor jako iterátor
    for c in cursor:
        idkey, brand, model, size, hres, vres, aspect, inputs, support, year, comments = c
        html += f'''    <tr>
            <td>{idkey}</td>
            <td>{brand}</td>
            <td>{model}</td>
            <td>{size}</td>
            <td>{hres}</td>
            <td>{vres}</td>
            <td>{aspect}</td>
            <td>{inputs}</td>
            <td>{support}</td>
            <td>{year}</td>
            <td>{comments}</td>
        </tr>'''

    # uzavření databáze
    cursor.close()
    if local_connection_flag == True:
        connection.close()
    
    # vrácení HTML
    return html

# sestavení SELECTu
def create_select(support, connection):
    cursor = connection.cursor()
    # 
    cursor.execute(f"""
        SELECT DISTINCT support FROM monitory;
    """)
    html = '<select name="support" form="monitory" onchange="submitFromJS()">'
    html += '<option value="">---</option>'
    for c in cursor:
        selected = 'selected' if support == c[0] else ''
        html += f'<option value="{c[0]}" {selected}>{c[0]}</option>'
    html += '</select>'
    # 
    cursor.close()
    # vrácení sestaveného SELECTu
    return html

# hlavní HTML-stránka
def create_html(brand, support):
    # připojení k SQLite-databázi
    connection = sqlite3.connect("./monitory.sqlite")
    cursor = connection.cursor()
    
    # návratové HTML
    html = ''

    # ovládací JavaScript
    html += '<script src="aplikace.js"></script>'
    
    # stylopis
    html += '''
    <style>
        #header { background-color: yellow; }
        tr:nth-child(2n+1) { background-color: lightblue; }
    </style>
    '''

    # začátek tabulky
    html += f'''<table border>
    <thead>
        <tr>
            <th colspan="11">
                <input type="submit" form="monitory">
            </th>
        </tr>
        <tr id="header">
            <th>id</th>
            <th>
                brand
                <form method="POST" id="monitory">
                    <input type="text" name="brand" value="{brand}">
                </form>
            </th>
            <th>model</th>
            <th>size</th>
            <th>hres</th>
            <th>vres</th>
            <th>aspect</th>
            <th>inputs</th>
            <th>
                support
                {create_select(support, connection)}
            </th>
            <th>year</th>
            <th>comments</th>
        </tr>
    </thead>
    '''

    # dotaz do databáze
    html += '<tbody>'
    html += get_monitors(brand, support, connection)
    html += '</tbody>'

    # konec tabulky
    html += '\n</table>'
    
    # <form onsubmit=""> již nefunguje kvůli výchozí CSP:
    #   https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Headers/Content-Security-Policy/script-src
    # Opis pomocí 'addEventListener()' kupodivu ale funkční je...
    html += '''
    <script>
        document.getElementById('monitory').addEventListener(
            "submit",
            function(event) {
                // Neodesílej formulář výchozím způsobem..
                event.preventDefault();
                // ..postará se o to následující funkce AJAXem:
                submitFromJS();
            }
        );
    </script>
    '''

    # uzavření databáze
    cursor.close()
    connection.close()
    
    # vrácení HTML
    return html

#
# WSGI-klient
#

# obsluha HTTP-dotazu "GET /"
def db_question(environ):
    # napársování dotazu
    cl_as_string = environ['CONTENT_LENGTH']                # počet bajtů v těle HTTP-requestu (jako řetězec -- je to HTTP-hlavička)
    cl = 0 if cl_as_string == '' else int(cl_as_string)     # při prvním otevření stránky (nebo prázdném dotazu) není v těle dotazu nic
    form = parse_qs(
        environ['wsgi.input'].read(cl).decode('utf-8')      # načteme z těla dotazu příslušný počet bajtů a převedeme je na řetězec
    )
    print('HTTP request:', form)
    q_brand = form['brand'][0] if 'brand' in form else ''
    q_support = form['support'][0] if 'support' in form else ''
    # vytvoření odpovědi
    body = bytes(create_html(q_brand, q_support), encoding='utf-8')
    return body

# obsluha HTTP-dotazu "GET /monitory"
def db_question_monitors(environ):
    # napársování dotazu
    cl_as_string = environ['CONTENT_LENGTH']                # počet bajtů v těle HTTP-requestu (jako řetězec -- je to HTTP-hlavička)
    cl = 0 if cl_as_string == '' else int(cl_as_string)     # při prvním otevření stránky (nebo prázdném dotazu) není v těle dotazu nic
    form = parse_qs(
        environ['wsgi.input'].read(cl).decode('utf-8')      # načteme z těla dotazu příslušný počet bajtů a převedeme je na řetězec
    )
    print('HTTP request:', form)
    q_brand = form['brand'][0] if 'brand' in form else ''
    q_support = form['support'][0] if 'support' in form else ''
    # vytvoření odpovědi
    body = bytes(get_monitors(q_brand, q_support), encoding='utf-8')
    return body

# hlavní aplikační funkce
# ~ odpovídá na dotazy na různé adresy (cesty) na běžícím serveru
def application(environ, start_response):
    #print(environ['PATH_INFO'])
    match environ['PATH_INFO']:
        case '/':
            body = db_question(environ)
            response_headers = [
                ("Content-type", "text/html; charset=UTF-8"),
                ("Content-length", str(len(body)) ),
            ]
            start_response("200 OK", response_headers)
            return (body,)
        case '/monitory':
            body = db_question_monitors(environ)
            response_headers = [
                ("Content-type", "text/html; charset=UTF-8"),
                ("Content-length", str(len(body)) ),
            ]
            start_response("200 OK", response_headers)
            return (body,)
        case '/aplikace.js':
            with open('aplikace.js') as f:
                body = f.read().encode('utf-8')
            response_headers = [
                ("Content-type", "text/javascript"),
                ("Content-length", str(len(body))),
            ]
            start_response("200 OK", response_headers)
            return (body,)
        case '/favicon.ico':
            response_headers = [
                ("Content-type", "image/x-icon"),
                ("Content-length", "0"),
            ]
            start_response("200 OK", response_headers)
            return (b'',)
        case _:
            print('_:', environ['PATH_INFO'])
            response_headers = [
                ("Content-type", "text/plain"),
                ("Content-length", "6"),
            ]
            start_response("200 OK", response_headers)
            return (b'200 OK',)

#
# WSGI-server (spuštěn natrvalo)
#
if __name__ == '__main__':
    from wsgiref.simple_server import make_server
    server = make_server('localhost', 8080, application)
    server.serve_forever()
